/* 
                      DO NOT EDIT THIS FILE!

 it has been automaticaly created by php4/scripts/credits from  
 the information found in the various php4/ext/.../CREDITS and
 php4/sapi/.../CREDITS files 
 
 if you want to change an entry you have to edit the appropriate 
 CREDITS file instead

*/

CREDIT_LINE("AOLserver","Sascha Schumann");
CREDIT_LINE("Apache 1.3","Rasmus Lerdorf, Zeev Suraski, Stig Bakken, David Sklar");
CREDIT_LINE("Apache 2.0","Sascha Schumann");
CREDIT_LINE("Caudium / Roxen","David Hedbor");
CREDIT_LINE("CGI","Rasmus Lerdorf, Stig Bakken");
CREDIT_LINE("fhttpd","Alex Belits");
CREDIT_LINE("ISAPI","Andi Gutmans, Zeev Suraski");
CREDIT_LINE("Java Servlet","Sam Ruby");
CREDIT_LINE("NSAPI","Jayakumar Muthukumarasamy");
CREDIT_LINE("phttpd","Thies C. Arntzen");
CREDIT_LINE("pi3web","Holger Zimmermann");
CREDIT_LINE("thttpd","Sascha Schumann");
