/* 
                      DO NOT EDIT THIS FILE!

 it has been automaticaly created by php4/scripts/credits from  
 the information found in the various php4/ext/.../CREDITS and
 php4/sapi/.../CREDITS files 
 
 if you want to change an entry you have to edit the appropriate 
 CREDITS file instead

*/

CREDIT_LINE("aspell","Rasmus Lerdorf");
CREDIT_LINE("Assert","Thies C. Arntzen");
CREDIT_LINE("BC Math","Andi Gutmans");
CREDIT_LINE("bz2","Sterling Hughes");
CREDIT_LINE("ccvs","Brendan W. McAdams, Doug DeJulio");
CREDIT_LINE("cpdf","Uwe Steinmann");
CREDIT_LINE("ctype","Hartmut Holzgraefe");
CREDIT_LINE("CURL","Sterling Hughes");
CREDIT_LINE("CyberCash","Evan Klinger");
CREDIT_LINE("DAV","Stig Bakken");
CREDIT_LINE("DBA","Sascha Schumann");
CREDIT_LINE("dBase","Jim Winstead");
CREDIT_LINE("DBM","Rasmus Lerdorf, Jim Winstead");
CREDIT_LINE("domxml","Uwe Steinmann");
CREDIT_LINE("dotnet","Sam Ruby");
CREDIT_LINE("EXIF","Rasmus Lerdorf");
CREDIT_LINE("FDF","Uwe Steinmann");
CREDIT_LINE("FilePro","Chad Robinson");
CREDIT_LINE("FTP","Andrew Skalski");
CREDIT_LINE("GD imaging","Rasmus Lerdorf, Stig Bakken, Jim Winstead, Jouni Ahto");
CREDIT_LINE("GetText","Alex Plotnick");
CREDIT_LINE("HyperWave","Uwe Steinmann");
CREDIT_LINE("icap","Mark Musone");
CREDIT_LINE("iisfunc","Frank M. Kromann");
CREDIT_LINE("IMAP","Rex Logan, Mark Musone, Brian Wang, Kaj-Michael Lang, Antoni Pamies Olive, Rasmus Lerdorf, Andrew Skalski, Chuck Hagenbuch");
CREDIT_LINE("Informix","Danny Heijl, Christian Cartus");
CREDIT_LINE("Ingres II","David Hénot");
CREDIT_LINE("InterBase","Jouni Ahto, Andrew Avdeev");
CREDIT_LINE("IRCG","Sascha Schumann");
CREDIT_LINE("Java","Sam Ruby");
CREDIT_LINE("LDAP","Amitay Isaacs, Eric Warnke, Rasmus Lerdorf, Gerrit Thomson");
CREDIT_LINE("MCAL","Mark Musone, Chuck Hagenbuch");
CREDIT_LINE("mcrypt","Sascha Schumann, Derick Rethans");
CREDIT_LINE("mhash","Sascha Schumann");
CREDIT_LINE("mSQL","Zeev Suraski");
CREDIT_LINE("MS SQL","Frank M. Kromann");
CREDIT_LINE("MySQL","Zeev Suraski");
CREDIT_LINE("OCI8","Stig Bakken, Thies C. Arntzen");
CREDIT_LINE("ODBC","Stig Bakken, Andreas Karajannis, Frank M. Kromann");
CREDIT_LINE("OpenSSL","Stig Venaas");
CREDIT_LINE("Oracle","Stig Bakken, Mitch Golden, Rasmus Lerdorf, Andreas Karajannis, Thies C. Arntzen");
CREDIT_LINE("Ovrimos","Nikos Mavroyanopoulos");
CREDIT_LINE("PDF","Uwe Steinmann");
CREDIT_LINE("Perl Compatible Regexps","Andrei Zmievski");
CREDIT_LINE("Posix","Kristian Köhntopp");
CREDIT_LINE("PostgreSQL","Jouni Ahto, Zeev Suraski");
CREDIT_LINE("Readline","Thies C. Arntzen");
CREDIT_LINE("Recode","Kristian Köhntopp");
CREDIT_LINE("Sablot","Sterling Hughes");
CREDIT_LINE("Satellite (CORBA)","David Eriksson");
CREDIT_LINE("Sessions","Sascha Schumann, Andrei Zmievski");
CREDIT_LINE("Shared Memory Operations","Slava Poliakov, Ilia Alshanetsky");
CREDIT_LINE("SNMP","Rasmus Lerdorf");
CREDIT_LINE("Sockets","Chris Vandomelen, Sterling Hughes");
CREDIT_LINE("SWF","Sterling Hughes");
CREDIT_LINE("Sybase-CT","Zeev Suraski, Tom May");
CREDIT_LINE("Sybase-DB","Zeev Suraski");
CREDIT_LINE("System V Semaphores","Tom May");
CREDIT_LINE("System V Shared Memory","Christian Cartus");
CREDIT_LINE("Verisign Payflow Pro","John Donagher, David Croft");
CREDIT_LINE("WDDX","Andrei Zmievski");
CREDIT_LINE("Win32 COM","Zeev Suraski");
CREDIT_LINE("XML","Stig Bakken, Thies C. Arntzen");
CREDIT_LINE("YAZ","Adam Dickmeiss");
CREDIT_LINE("Yellow Pages","Stephanie Wehner");
CREDIT_LINE("Zlib","Rasmus Lerdorf, Stefan Roehrich");
CREDIT_LINE("zZIPlib","Hartmut Holzgraefe");
