#ifndef YYSTYPE
#define YYSTYPE int
#endif
#define	T_INCLUDE	257
#define	T_INCLUDE_ONCE	258
#define	T_EVAL	259
#define	T_REQUIRE	260
#define	T_REQUIRE_ONCE	261
#define	T_LOGICAL_OR	262
#define	T_LOGICAL_XOR	263
#define	T_LOGICAL_AND	264
#define	T_PRINT	265
#define	T_PLUS_EQUAL	266
#define	T_MINUS_EQUAL	267
#define	T_MUL_EQUAL	268
#define	T_DIV_EQUAL	269
#define	T_CONCAT_EQUAL	270
#define	T_MOD_EQUAL	271
#define	T_AND_EQUAL	272
#define	T_OR_EQUAL	273
#define	T_XOR_EQUAL	274
#define	T_SL_EQUAL	275
#define	T_SR_EQUAL	276
#define	T_BOOLEAN_OR	277
#define	T_BOOLEAN_AND	278
#define	T_IS_EQUAL	279
#define	T_IS_NOT_EQUAL	280
#define	T_IS_IDENTICAL	281
#define	T_IS_NOT_IDENTICAL	282
#define	T_IS_SMALLER_OR_EQUAL	283
#define	T_IS_GREATER_OR_EQUAL	284
#define	T_SL	285
#define	T_SR	286
#define	T_INC	287
#define	T_DEC	288
#define	T_INT_CAST	289
#define	T_DOUBLE_CAST	290
#define	T_STRING_CAST	291
#define	T_ARRAY_CAST	292
#define	T_OBJECT_CAST	293
#define	T_BOOL_CAST	294
#define	T_UNSET_CAST	295
#define	T_NEW	296
#define	T_EXIT	297
#define	T_IF	298
#define	T_ELSEIF	299
#define	T_ELSE	300
#define	T_ENDIF	301
#define	T_LNUMBER	302
#define	T_DNUMBER	303
#define	T_STRING	304
#define	T_STRING_VARNAME	305
#define	T_VARIABLE	306
#define	T_NUM_STRING	307
#define	T_INLINE_HTML	308
#define	T_CHARACTER	309
#define	T_BAD_CHARACTER	310
#define	T_ENCAPSED_AND_WHITESPACE	311
#define	T_CONSTANT_ENCAPSED_STRING	312
#define	T_ECHO	313
#define	T_DO	314
#define	T_WHILE	315
#define	T_ENDWHILE	316
#define	T_FOR	317
#define	T_ENDFOR	318
#define	T_FOREACH	319
#define	T_ENDFOREACH	320
#define	T_DECLARE	321
#define	T_ENDDECLARE	322
#define	T_AS	323
#define	T_SWITCH	324
#define	T_ENDSWITCH	325
#define	T_CASE	326
#define	T_DEFAULT	327
#define	T_BREAK	328
#define	T_CONTINUE	329
#define	T_OLD_FUNCTION	330
#define	T_FUNCTION	331
#define	T_CONST	332
#define	T_RETURN	333
#define	T_USE	334
#define	T_GLOBAL	335
#define	T_STATIC	336
#define	T_VAR	337
#define	T_UNSET	338
#define	T_ISSET	339
#define	T_EMPTY	340
#define	T_CLASS	341
#define	T_EXTENDS	342
#define	T_OBJECT_OPERATOR	343
#define	T_DOUBLE_ARROW	344
#define	T_LIST	345
#define	T_ARRAY	346
#define	T_LINE	347
#define	T_FILE	348
#define	T_COMMENT	349
#define	T_ML_COMMENT	350
#define	T_OPEN_TAG	351
#define	T_OPEN_TAG_WITH_ECHO	352
#define	T_CLOSE_TAG	353
#define	T_WHITESPACE	354
#define	T_START_HEREDOC	355
#define	T_END_HEREDOC	356
#define	T_DOLLAR_OPEN_CURLY_BRACES	357
#define	T_CURLY_OPEN	358
#define	T_PAAMAYIM_NEKUDOTAYIM	359

